/*
** wifimgr-gtk.c
*/

/*
** Copyright (c) 2009, J.R. Oldroyd, Open Advisors Limited
** All rights reserved.
** 
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the author, the author's organization nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
** 
** THIS SOFTWARE IS PROVIDED BY OPEN ADVISORS LIMITED ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL OPEN ADVISORS LIMITED BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
** $Id: wifimgr-gtk.c 57 2009-12-18 19:29:30Z jr $
*/

#include "wifimgr.h"
#include "version.h"
#include <gtk/gtk.h>
#include <string.h>

#ifndef WITHOUT_NLS
#include <libintl.h>
#else
#define gettext(x)	(x)
#endif

#define	ALIGN_LEFT	0
#define	ALIGN_CENTER	0.5
#define	ALIGN_RIGHT	1
#define	ALIGN_TOP	0
#define	ALIGN_MIDDLE	0.5
#define	ALIGN_BOTTOM	1

char		gui_response[1024];
int		gui_changes;

/*
** initialize GUI
*/
int
gui_init(int * ac, char *** av) {
	return gtk_init_check(ac, av);
}

/*
** handle check_button input by user
*/
static void
gui_process_check_button(GtkWidget * w, gpointer * gp) {
	int			check;

	check = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	*((int *) gp) = check;
	gui_changes = 1;
}

/*
** handle spin button input by user
*/
static void
gui_process_spin_button(GtkWidget * w, gpointer * gp) {
	int			val;

	val = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(w));
	*((int *) gp) = val;
	gui_changes = 1;
}

/*
** handle text input by user
*/
static void
gui_process_text_input(GtkWidget * w, gpointer * gp) {
	char *			text;

	text = (char *) gtk_entry_get_text(GTK_ENTRY(w));
	strcpy((char *) gp, text);
	gui_changes = 1;
}

/*
** display GUI message dialog with OK button
*/
int
gui_message(char * s, int msg_type) {
	GtkWidget *		dialog;
	GtkWidget *		w;
	int			resp;

	switch(msg_type) {
	case MSG_INFO:
		dialog = gtk_message_dialog_new_with_markup(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
		    GTK_MESSAGE_INFO, GTK_BUTTONS_OK, s);
		break;
	case MSG_WARNING:
		dialog = gtk_message_dialog_new_with_markup(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
		    GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, s);
		break;
	case MSG_QUESTION:
		dialog = gtk_message_dialog_new_with_markup(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
		    GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, s);
		break;
	case MSG_ERROR:
		dialog = gtk_message_dialog_new_with_markup(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
		    GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, s);
		break;
	case MSG_INPUT:
		dialog = gtk_message_dialog_new_with_markup(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
		    GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK, s);
		w = gtk_entry_new();
		gtk_entry_set_visibility(GTK_ENTRY(w), FALSE);
		gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
		gtk_entry_set_activates_default(GTK_ENTRY(w), TRUE); /* allow CR to end input */
		g_signal_connect(G_OBJECT(w), "changed", G_CALLBACK(gui_process_text_input),
		    (gpointer) gui_response);
		gtk_container_add(GTK_CONTAINER(GTK_MESSAGE_DIALOG(dialog)->label->parent), w);
		gtk_widget_show_all(dialog);
		break;
	}
	gtk_window_set_title(GTK_WINDOW(dialog), gettext("WiFi Networks Manager"));

	resp = gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	switch(resp) {
	case GTK_RESPONSE_OK:	return MSG_RESPONSE_OK;
	case GTK_RESPONSE_YES:	return MSG_RESPONSE_YES;
	case GTK_RESPONSE_NO:	return MSG_RESPONSE_NO;
	default:		return MSG_RESPONSE_FAIL;
	}
}

/*
** delete existing table data, if any
** fill table with network data
*/
GtkWidget *
gui_fill_network_table(GtkWidget * x, gpointer * gp) {
	GtkWidget *		table;
	GtkWidget *		table_viewport = NULL;
	GtkWidget *		w;
	int			row;
	int			col;
	struct wifi_net *	net;
	char *			associated_ssid;
	char *			icon;
	char			buf[256];

	if (gp) {
		if (gui_changes && gui_message(gettext("Discard changes?"), MSG_QUESTION) != MSG_RESPONSE_YES)
			return NULL;

		table = *((GtkWidget **) gp);
		table_viewport = table->parent;
		gtk_widget_destroy(table);
	}

	gui_changes = 0;

	table = gtk_table_new(1, 1, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 2);

	row = 0;
	col = 0;

	w = gtk_label_new(gettext("<b>Enabled</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	w = gtk_label_new(gettext("<b>SSID</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_misc_set_alignment(GTK_MISC(w), ALIGN_LEFT, ALIGN_MIDDLE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	/* encryption */
	col++;

	w = gtk_label_new(gettext("<b>Encryption Key</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_misc_set_alignment(GTK_MISC(w), ALIGN_LEFT, ALIGN_MIDDLE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	w = gtk_label_new(gettext("<b>Comment</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_misc_set_alignment(GTK_MISC(w), ALIGN_LEFT, ALIGN_MIDDLE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	w = gtk_label_new(gettext("<b>Priority</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_misc_set_alignment(GTK_MISC(w), ALIGN_LEFT, ALIGN_MIDDLE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	w = gtk_label_new(gettext("<b>BSSID</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_misc_set_alignment(GTK_MISC(w), ALIGN_CENTER, ALIGN_MIDDLE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	w = gtk_label_new(gettext("<b>Chan</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_misc_set_alignment(GTK_MISC(w), ALIGN_RIGHT, ALIGN_MIDDLE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	w = gtk_label_new(gettext("<b>Mbps</b>"));
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_misc_set_alignment(GTK_MISC(w), ALIGN_RIGHT, ALIGN_MIDDLE);
	gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
	    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
	col++;

	row++;
	col = 0;

	associated_ssid = ifconfig_associated_network(wifi_if);

	for (net = build_network_list(wifi_if); net; net = net->wn_next) {
		col = 0;

		w = gtk_check_button_new();
		if (net->wn_enabled)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
		g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(gui_process_check_button),
		    (gpointer) &(net->wn_enabled));
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		if (strcmp(net->wn_ssid, associated_ssid) == 0) {
			w = gtk_image_new_from_file(ICON_PATH "/wifimgr24green.png");
			gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		}
		col++;

		w = gtk_label_new(net->wn_ssid);
		gtk_misc_set_alignment(GTK_MISC(w), ALIGN_LEFT, ALIGN_MIDDLE);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		switch (net->wn_security) {
		case WN_SEC_NONE:	icon = ICON_PATH "/padlock-open.png"; break;
		case WN_SEC_WEP:	icon = ICON_PATH "/padlock-wep.png"; break;
		case WN_SEC_WPA_PSK:	icon = ICON_PATH "/padlock-wpa.png"; break;
		case WN_SEC_OTHER:	icon = ICON_PATH "/padlock-unk.png"; break;
		}
		if (icon) {
			w = gtk_image_new_from_file(icon);
			gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		}
		col++;

		w = gtk_entry_new();
		gtk_entry_set_text(GTK_ENTRY(w), net->wn_key);
		if (net->wn_security == WN_SEC_NONE || net->wn_security == WN_SEC_OTHER)
			gtk_editable_set_editable(GTK_EDITABLE(w), FALSE);
		g_signal_connect(G_OBJECT(w), "changed", G_CALLBACK(gui_process_text_input),
		    (gpointer) net->wn_key);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		w = gtk_entry_new();
		gtk_entry_set_text(GTK_ENTRY(w), net->wn_comment);
		g_signal_connect(G_OBJECT(w), "changed", G_CALLBACK(gui_process_text_input),
		    (gpointer) net->wn_comment);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		w = gtk_spin_button_new_with_range(-1.0, 255.0, 1.0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), (gdouble) net->wn_priority);
		g_signal_connect(G_OBJECT(w), "value-changed", G_CALLBACK(gui_process_spin_button),
		    (gpointer) &net->wn_priority);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		w = gtk_label_new(net->wn_bssid);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		sprintf(buf, "%d", net->wn_chan);
		w = gtk_label_new(buf);
		gtk_misc_set_alignment(GTK_MISC(w), ALIGN_RIGHT, ALIGN_MIDDLE);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		sprintf(buf, "%d", net->wn_rate);
		w = gtk_label_new(buf);
		gtk_misc_set_alignment(GTK_MISC(w), ALIGN_RIGHT, ALIGN_MIDDLE);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		if (net->wn_bars > 80)      icon = ICON_PATH "/signal_bars_5.png";
		else if (net->wn_bars > 60) icon = ICON_PATH "/signal_bars_4.png";
		else if (net->wn_bars > 40) icon = ICON_PATH "/signal_bars_3.png";
		else if (net->wn_bars > 20) icon = ICON_PATH "/signal_bars_2.png";
		else if (net->wn_bars > 0)  icon = ICON_PATH "/signal_bars_1.png";
		else			    icon = ICON_PATH "/signal_bars_0.png";
		w = gtk_image_new_from_file(icon);
		gtk_table_attach(GTK_TABLE(table), w, col, col+1, row, row+1,
		    GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
		col++;

		row++;
		col = 0;
	}

	if (gp) {
		gtk_container_add(GTK_CONTAINER(table_viewport), table);
		gtk_widget_show_all(table);
		*gp = table;
	}

	return table;
}

/*
** main GUI loop
*/
void
gui_loop() {
	GtkWidget *		window;
	GtkWidget *		main_vbox;
	GtkWidget *		top_graphic;
	GtkWidget *		table_scrolled_window;
	GtkWidget *		bottom_buttons;
	GtkWidget *		table;
	GtkWidget *		w;
	char			buf[256];

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	gtk_window_set_default_size(GTK_WINDOW(window), 960, 480);
	gtk_window_set_title(GTK_WINDOW(window), gettext("WiFi Networks Manager"));
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
	g_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

	main_vbox = gtk_vbox_new(FALSE, 5);

	top_graphic = gtk_hbox_new(FALSE, 0);
	w = gtk_image_new_from_file(ICON_PATH "/freebsd-surf.png");
	gtk_box_pack_start(GTK_BOX(top_graphic), w, FALSE, FALSE, 0);
	w = gtk_image_new_from_file(ICON_PATH "/wifimgr.png");
	gtk_box_pack_end(GTK_BOX(top_graphic), w, FALSE, FALSE, 0);
	sprintf(buf, "<span size=\"x-large\"><b>%s</b></span>\n%s", gettext("WiFi Networks Manager"), VERSION);
	w = gtk_label_new(buf);
	gtk_label_set_use_markup(GTK_LABEL(w), TRUE);
	gtk_box_pack_end(GTK_BOX(top_graphic), w, FALSE, FALSE, 10);
	gtk_box_pack_start(GTK_BOX(main_vbox), top_graphic, FALSE, FALSE, 0);

	table_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(table_scrolled_window), 5);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(table_scrolled_window),
	    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	table = gui_fill_network_table(NULL, NULL);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(table_scrolled_window), table);
	gtk_box_pack_start(GTK_BOX(main_vbox), table_scrolled_window, TRUE, TRUE, 0);

	bottom_buttons = gtk_hbox_new(FALSE, 0);
	w = gtk_button_new_with_label(gettext("Rescan Networks"));
	g_signal_connect(GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(gui_fill_network_table),
	    (gpointer) &table);
	gtk_box_pack_start(GTK_BOX(bottom_buttons), w, FALSE, FALSE, 5);
	w = gtk_button_new_with_label(gettext("Close"));
	g_signal_connect(GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
	gtk_box_pack_end(GTK_BOX(bottom_buttons), w, FALSE, FALSE, 5);
	w = gtk_button_new_with_label(gettext("Save and Reconnect"));
	g_signal_connect(GTK_OBJECT(w), "clicked", GTK_SIGNAL_FUNC(save_networks_file), NULL);
	gtk_box_pack_end(GTK_BOX(bottom_buttons), w, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(main_vbox), bottom_buttons, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(window), main_vbox);

	gtk_widget_show_all(window);

	gtk_main();
}
